@@ -1,4 +1,5 @@
 import os
+import shutil
 from pathlib import Path
 import yaml
 
@@ -92,7 +93,7 @@ class BuilderModel:
 
                 # Build model
                 for s in self.software:
-                    if s == 'LEDET':    self.buildLEDET()
+                    if s == 'LEDET':    self.buildLEDET(case_model=self.case_model)
                     if s == 'SIGMA':    self.buildSIGMA()
                     if s == 'ProteCCT': self.buildProteCCT()
 
@@ -143,6 +144,7 @@ class BuilderModel:
 
                 # Build model
                 for s in self.software:
+                    if s == 'LEDET':    self.buildLEDET(case_model=self.case_model)
                     if s == 'PyBBQ':    self.buildPyBBQ()
 
 
@@ -260,90 +262,149 @@ class BuilderModel:
         roxie_parser = ParserRoxie()
         self.roxie_data = roxie_parser.getData(dir_data=self.path_data, dir_cadata=self.path_cadata, dir_iron=self.path_iron)
 
-    def buildLEDET(self):
+    def buildLEDET(self, case_model: str = 'magnet'):
         """
             Building a LEDET model
+            case_model is a string defining the type of model to build (magnet or conductor)
         """
-        magnet_name = self.model_data.GeneralParameters.magnet_name
-        nameFileSMIC = os.path.join(self.output_path, magnet_name + '_selfMutualInductanceMatrix.csv')  # full path of the .csv file with self-mutual inductances to write
 
-        # Copy/edit the ROXIE map2d file
-        suffix = "_All"
-        if self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 1:
-            #     # [[...half_turn_length, Ribbon...n_strands],.....]
-            # TODO: geometry when conductor has a combination of ribbon and non-ribbon cables
+        if case_model == 'magnet':
+            magnet_name = self.model_data.GeneralParameters.magnet_name
+            nameFileSMIC = os.path.join(self.output_path, magnet_name + '_selfMutualInductanceMatrix.csv')  # full path of the .csv file with self-mutual inductances to write
 
-            # List of flags that are True is the cable type is "Ribbon"
-            list_flag_ribbon = []
-            for i, cond in enumerate(self.model_data.CoilWindings.conductor_to_group):
-                list_flag_ribbon.append(self.model_data.Conductors[cond-1].cable.type == 'Ribbon')
+            # Copy/edit the ROXIE map2d file
+            suffix = "_All"
+            if self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 1:
+                #     # [[...half_turn_length, Ribbon...n_strands],.....]
+                # TODO: geometry when conductor has a combination of ribbon and non-ribbon cables
 
-            nT_from_original_map2d, nStrands_inGroup_original_map2d, _, _, _, _, _, _ = getParametersFromMap2d(
-                map2dFile=self.path_map2d, headerLines=self.model_data.Options_LEDET.field_map_files.headerLines,
-                verbose=self.verbose)
+                # List of flags that are True is the cable type is "Ribbon"
+                list_flag_ribbon = []
+                for i, cond in enumerate(self.model_data.CoilWindings.conductor_to_group):
+                    list_flag_ribbon.append(self.model_data.Conductors[cond-1].cable.type == 'Ribbon')
 
-            n_groups_original_file = len(nT_from_original_map2d)
-            geometry_ribbon_cable = []
+                nT_from_original_map2d, nStrands_inGroup_original_map2d, _, _, _, _, _, _ = getParametersFromMap2d(
+                    map2dFile=self.path_map2d, headerLines=self.model_data.Options_LEDET.field_map_files.headerLines,
+                    verbose=self.verbose)
 
-            for i in range(n_groups_original_file):
-                list = [None, None]
-                list[0] = int(nStrands_inGroup_original_map2d[i])  # layers
-                list[1] = nT_from_original_map2d[i]  # number of half-turns; in case it is not a ribbon cable, it is going to be ignored in the modify-ribbon-cable function
-                geometry_ribbon_cable.append(list)
+                n_groups_original_file = len(nT_from_original_map2d)
+                geometry_ribbon_cable = []
 
-            if self.verbose:
-                print('geometry_ribbon_cable: {}'.format(geometry_ribbon_cable))
+                for i in range(n_groups_original_file):
+                    list = [None, None]
+                    list[0] = int(nStrands_inGroup_original_map2d[i])  # layers
+                    list[1] = nT_from_original_map2d[i]  # number of half-turns; in case it is not a ribbon cable, it is going to be ignored in the modify-ribbon-cable function
+                    geometry_ribbon_cable.append(list)
 
-            file_name_output = copy_modified_map2d_ribbon_cable(magnet_name,
-                                                                self.path_map2d,
-                                                                self.output_path, geometry_ribbon_cable,
-                                                                self.model_data.Options_LEDET.field_map_files.flagIron,
-                                                                self.model_data.Options_LEDET.field_map_files.flagSelfField,
-                                                                list_flag_ribbon,
-                                                                suffix=suffix, verbose=self.verbose)
+                if self.verbose:
+                    print('geometry_ribbon_cable: {}'.format(geometry_ribbon_cable))
 
-        elif self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 0 or self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == None:
-            file_name_output = copy_map2d(magnet_name, self.path_map2d,
-                                          self.output_path, self.model_data.Options_LEDET.field_map_files.flagIron,
-                                          self.model_data.Options_LEDET.field_map_files.flagSelfField, suffix=suffix,
-                                          verbose=self.verbose)
+                file_name_output = copy_modified_map2d_ribbon_cable(magnet_name,
+                                                                    self.path_map2d,
+                                                                    self.output_path, geometry_ribbon_cable,
+                                                                    self.model_data.Options_LEDET.field_map_files.flagIron,
+                                                                    self.model_data.Options_LEDET.field_map_files.flagSelfField,
+                                                                    list_flag_ribbon,
+                                                                    suffix=suffix, verbose=self.verbose)
 
-        self.map2d_file_modified = os.path.join(self.output_path, file_name_output)
+            elif self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 0 or self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == None:
+                file_name_output = copy_map2d(magnet_name, self.path_map2d,
+                                              self.output_path, self.model_data.Options_LEDET.field_map_files.flagIron,
+                                              self.model_data.Options_LEDET.field_map_files.flagSelfField, suffix=suffix,
+                                              verbose=self.verbose)
 
-        builder_ledet = BuilderLEDET(path_magnet=self.path_magnet, input_model_data=self.model_data,
-                                     input_roxie_data=self.roxie_data, input_map2d=self.map2d_file_modified,
-                                     smic_write_path=nameFileSMIC, flag_build=self.flag_build, flag_plot_all=self.flag_plot_all,
-                                     verbose=self.verbose)
+            self.map2d_file_modified = os.path.join(self.output_path, file_name_output)
 
-        # Copy or modify+copy magnet-name_E....map2d files
-        number_input_files = len([entry for entry in os.listdir(self.path_magnet) if os.path.isfile(os.path.join(self.path_magnet, entry))])
-        for file in range(number_input_files+1):
-            suffix = '_E{}'.format(file)
-            path_map2d_E = os.path.join(self.path_magnet, magnet_name + suffix + '.map2d')
-            if os.path.isfile(path_map2d_E):
-                if self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 1:
-                    copy_modified_map2d_ribbon_cable(magnet_name,
-                                                     path_map2d_E,
-                                                     self.output_path, geometry_ribbon_cable,
-                                                     self.model_data.Options_LEDET.field_map_files.flagIron,
-                                                     self.model_data.Options_LEDET.field_map_files.flagSelfField,
-                                                     list_flag_ribbon,
-                                                     suffix=suffix, verbose=self.verbose)
-                elif self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 0 or self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == None:
-                    copy_map2d(magnet_name, path_map2d_E, self.output_path, self.model_data.Options_LEDET.field_map_files.flagIron,
-                                          self.model_data.Options_LEDET.field_map_files.flagSelfField, suffix=suffix,
-                                          verbose=self.verbose)
+            # Copy the additional geometry and magnetic field csv file, if defined in the input file
+            if self.model_data.Options_LEDET.simulation_3D.sim3D_flag_Import3DGeometry == 1:
+                name_geometry_csv_file = magnet_name + '_' + str(self.conductor_data.Options_LEDET.simulation_3D.sim3D_import3DGeometry_modelNumber) + '.csv'
+                input_path_full  = os.path.join(self.path_magnet, name_geometry_csv_file)
+                output_path_full = os.path.join(self.output_path, name_geometry_csv_file)
+                shutil.copy2(input_path_full, output_path_full)
+                if self.verbose:
+                    print('File {} copied to {}.'.format(input_path_full, output_path_full))
 
-        # Write output excel file
-        parser_ledet = ParserLEDET(builder_ledet)
-        nameFileLEDET = os.path.join(self.output_path, magnet_name + '.xlsx')  # full path of the LEDET input file to write
-        parser_ledet.writeLedet2Excel(full_path_file_name=nameFileLEDET, verbose=self.verbose, SkipConsistencyCheck=True)
+            builder_ledet = BuilderLEDET(path_magnet=self.path_magnet, input_model_data=self.model_data,
+                                         input_roxie_data=self.roxie_data, input_map2d=self.map2d_file_modified,
+                                         smic_write_path=nameFileSMIC, flag_build=self.flag_build, flag_plot_all=self.flag_plot_all,
+                                         verbose=self.verbose,
+                                         case_model=case_model)
 
-        # Write output json file
-        if self.flag_json:
-            nameFileLedetJson = os.path.join(self.output_path, magnet_name + '.json')  # full path of the LEDET input file to write
-            parser_ledet.write2json(full_path_file_name=nameFileLedetJson, verbose=self.verbose, SkipConsistencyCheck=True)
+            # Copy or modify+copy magnet-name_E....map2d files
+            number_input_files = len([entry for entry in os.listdir(self.path_magnet) if os.path.isfile(os.path.join(self.path_magnet, entry))])
+            for file in range(number_input_files+1):
+                suffix = '_E{}'.format(file)
+                path_map2d_E = os.path.join(self.path_magnet, magnet_name + suffix + '.map2d')
+                if os.path.isfile(path_map2d_E):
+                    if self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 1:
+                        copy_modified_map2d_ribbon_cable(magnet_name,
+                                                         path_map2d_E,
+                                                         self.output_path, geometry_ribbon_cable,
+                                                         self.model_data.Options_LEDET.field_map_files.flagIron,
+                                                         self.model_data.Options_LEDET.field_map_files.flagSelfField,
+                                                         list_flag_ribbon,
+                                                         suffix=suffix, verbose=self.verbose)
+                    elif self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 0 or self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == None:
+                        copy_map2d(magnet_name, path_map2d_E, self.output_path, self.model_data.Options_LEDET.field_map_files.flagIron,
+                                              self.model_data.Options_LEDET.field_map_files.flagSelfField, suffix=suffix,
+                                              verbose=self.verbose)
 
+            # Write output excel file
+            parser_ledet = ParserLEDET(builder_ledet)
+            nameFileLEDET = os.path.join(self.output_path, magnet_name + '.xlsx')  # full path of the LEDET input file to write
+            parser_ledet.writeLedet2Excel(full_path_file_name=nameFileLEDET, verbose=self.verbose, SkipConsistencyCheck=True)
+
+            # Write output json file
+            if self.flag_json:
+                nameFileLedetJson = os.path.join(self.output_path, magnet_name + '.json')  # full path of the LEDET input file to write
+                parser_ledet.write2json(full_path_file_name=nameFileLedetJson, verbose=self.verbose, SkipConsistencyCheck=True)
+
+        elif case_model == 'conductor':
+            conductor_name = self.conductor_data.GeneralParameters.conductor_name
+
+            # Copy the ROXIE map2d file, if defined in the input file
+            if self.path_map2d:
+                suffix = "_All"
+                file_name_output = copy_map2d(conductor_name,
+                                              self.path_map2d,
+                                              self.output_path,
+                                              self.conductor_data.Options_LEDET.field_map_files.flagIron,
+                                              self.conductor_data.Options_LEDET.field_map_files.flagSelfField,
+                                              suffix=suffix,
+                                              verbose=self.verbose)
+                self.map2d_file_modified = os.path.join(self.output_path, file_name_output)
+            else:
+                self.map2d_file_modified = None
+                if self.verbose:
+                    print('Map2d file {} not present, hence it will not be copied.'.format(self.path_map2d))
+
+            # Copy the additional geometry and magnetic field csv file, if defined in the input file
+            if self.conductor_data.Options_LEDET.simulation_3D.sim3D_flag_Import3DGeometry == 1:
+                name_geometry_csv_file = conductor_name + '_' + str(self.conductor_data.Options_LEDET.simulation_3D.sim3D_import3DGeometry_modelNumber) + '.csv'
+                input_path_full  = os.path.join(self.path_magnet, name_geometry_csv_file)
+                output_path_full = os.path.join(self.output_path, name_geometry_csv_file)
+                shutil.copy2(input_path_full, output_path_full)
+                if self.verbose:
+                    print('File {} copied to {}.'.format(input_path_full, output_path_full))
+
+            builder_ledet = BuilderLEDET(path_magnet=self.path_magnet, input_model_data=self.conductor_data,
+                                         input_map2d=self.map2d_file_modified,
+                                         flag_build=self.flag_build, flag_plot_all=self.flag_plot_all,
+                                         verbose=self.verbose,
+                                         case_model=case_model)
+
+            # Write output excel file
+            parser_ledet = ParserLEDET(builder_ledet)
+            nameFileLEDET = os.path.join(self.output_path, conductor_name + '.xlsx')  # full path of the LEDET input file to write
+            parser_ledet.writeLedet2Excel(full_path_file_name=nameFileLEDET, verbose=self.verbose, SkipConsistencyCheck=True)
+
+            # Write output json file
+            if self.flag_json:
+                nameFileLedetJson = os.path.join(self.output_path, conductor_name + '.json')  # full path of the LEDET input file to write
+                parser_ledet.write2json(full_path_file_name=nameFileLedetJson, verbose=self.verbose, SkipConsistencyCheck=True)
+
+        else:
+            raise Exception('Case model {} is not supported when building a LEDET model.'.format(case_model))
 
     def buildProteCCT(self):
         """
