@@ -131,49 +131,50 @@ class LEDETInputs:
 class LEDETOptions:
     time_vector_params: np.ndarray = np.array([])
     Iref: float = None
-    flagIron: float = None
-    flagSelfField: float = None
-    headerLines: float = None
+    flagIron: int = None
+    flagSelfField: int = None
+    headerLines: int = None
     columnsXY: np.ndarray = np.array([])
     columnsBxBy: np.ndarray = np.array([])
-    flagPlotMTF: float = None
-    flag_typeWindings: float = None
-    flag_calculateInductanceMatrix: float = None
-    flag_useExternalInitialization: float = None
-    flag_initializeVar: float = None
-    flag_fastMode: float = None
-    flag_controlCurrent: float = None
-    flag_automaticRefinedTimeStepping: float = None
-    flag_IronSaturation: float = None
-    flag_InvertCurrentsAndFields: float = None
-    flag_ScaleDownSuperposedMagneticField: float = None
-    flag_HeCooling: float = None
+    flagPlotMTF: int = None
+    fieldMapNumber: int = None
+    flag_typeWindings: int = None
+    flag_calculateInductanceMatrix: int = None
+    flag_useExternalInitialization: int = None
+    flag_initializeVar: int = None
+    flag_fastMode: int = None
+    flag_controlCurrent: int = None
+    flag_automaticRefinedTimeStepping: int = None
+    flag_IronSaturation: int = None
+    flag_InvertCurrentsAndFields: int = None
+    flag_ScaleDownSuperposedMagneticField: int = None
+    flag_HeCooling: int = None
     fScaling_Pex: float = None
     fScaling_Pex_AlongHeight: float = None
     fScaling_MR: float = None
-    flag_scaleCoilResistance_StrandTwistPitch: float = None
-    flag_separateInsulationHeatCapacity: float = None
-    flag_persistentCurrents: float = None
-    flag_ISCL: float = None
+    flag_scaleCoilResistance_StrandTwistPitch: int = None
+    flag_separateInsulationHeatCapacity: int = None
+    flag_persistentCurrents: int = None
+    flag_ISCL: int = None
     fScaling_Mif: float = None
     fScaling_Mis: float = None
-    flag_StopIFCCsAfterQuench: float = None
-    flag_StopISCCsAfterQuench: float = None
+    flag_StopIFCCsAfterQuench: int = None
+    flag_StopISCCsAfterQuench: int = None
     tau_increaseRif: float = None
     tau_increaseRis: float = None
     fScaling_RhoSS: float = None
     maxVoltagePC: float = None
     minCurrentDiode: float = None
-    flag_symmetricGroundingEE: float = None
-    flag_removeUc: float = None
+    flag_symmetricGroundingEE: int = None
+    flag_removeUc: int = None
     BtX_background: float = None
     BtY_background: float = None
-    flag_showFigures: float = None
-    flag_saveFigures: float = None
-    flag_saveMatFile: float = None
-    flag_saveTxtFiles: float = None
-    flag_generateReport: float = None
-    flag_hotSpotTemperatureInEachGroup: float = None
+    flag_showFigures: int = None
+    flag_saveFigures: int = None
+    flag_saveMatFile: int = None
+    flag_saveTxtFiles: int = None
+    flag_generateReport: int = None
+    flag_hotSpotTemperatureInEachGroup: int = None
     flag_3D: int = None
     flag_adaptiveTimeStepping: int = None
     sim3D_flag_Import3DGeometry: int = None
