@@ -956,6 +956,9 @@ def copy_map2d(magnet_name: str, map2d_file_name: str, output_path: str, flagIro
     # Copy
     shutil.copy2(map2d_file_name, output_path_full)
 
+    if verbose:
+        print('File {} copied to {}.'.format(map2d_file_name, output_path_full))
+
     return file_name_output
 
 
