@@ -71,6 +71,78 @@ def evalContent(s, g):
     return value
 
 
+def get_conductor_corners(roxie_data: pd.APIdata):
+    """ Returns conductor corner positions (insulated and bare)  and conductor currents in list form """
+    x_insulated = []
+    y_insulated = []
+    x_bare = []
+    y_bare = []
+    i_conductor = []
+    x_strand = []
+    y_strand = []
+    i_strand = []
+    strand_to_halfTurn = []
+    idx_half_turn = 0
+    for eo in roxie_data.coil.electrical_order:
+        winding = roxie_data.coil.coils[eo.coil].poles[eo.pole].layers[eo.layer].windings[eo.winding]
+        block = winding.blocks[eo.block]
+        for halfTurn_nr, halfTurn in block.half_turns.items():
+            idx_half_turn = idx_half_turn + 1
+            insulated = halfTurn.corners.insulated
+            bare = halfTurn.corners.bare
+            x_insulated.append([insulated.iL.x, insulated.oL.x, insulated.oR.x, insulated.iR.x])
+            y_insulated.append([insulated.iL.y, insulated.oL.y, insulated.oR.y, insulated.iR.y])
+            x_bare.append([bare.iL.x, bare.oL.x, bare.oR.x, bare.iR.x])
+            y_bare.append([bare.iL.y, bare.oL.y, bare.oR.y, bare.iR.y])
+            i_conductor.append(winding.cable_current * block.current_sign)
+
+            for strand_gr_nr, strand_gr in halfTurn.strand_groups.items():
+                for strand_nr, strand in strand_gr.strand_positions.items():
+                    x_strand.append(strand.x)
+                    y_strand.append(strand.y)
+                    i_strand.append(winding.strand_current * block.current_sign)
+                    strand_to_halfTurn.append(idx_half_turn)
+
+    return x_insulated, y_insulated, x_bare, y_bare, i_conductor, x_strand, y_strand, i_strand, strand_to_halfTurn
+
+
+def get_conductor_corners_old(roxie_data: pd.APIdata):
+    """ Returns conductor corner positions (insulated and bare)  and conductor currents in list form """
+    x_insulated = []
+    y_insulated = []
+    x_bare = []
+    y_bare = []
+    i_conductor = []
+    x_strand = []
+    y_strand = []
+    i_strand = []
+    strandToHalfTurn = []
+    idx_half_turn = 0
+    for coil_nr, coil in roxie_data.coil.coils.items():
+        for pole_nr, pole in coil.poles.items():
+            for layer_nr, layer in pole.layers.items():
+                for winding_key, winding in layer.windings.items():
+                    for block_key, block in winding.blocks.items():
+                        for halfTurn_nr, halfTurn in block.half_turns.items():
+                            idx_half_turn = idx_half_turn + 1
+                            insulated = halfTurn.corners.insulated
+                            bare = halfTurn.corners.bare
+                            x_insulated.append([insulated.iL.x, insulated.oL.x, insulated.oR.x, insulated.iR.x])
+                            y_insulated.append([insulated.iL.y, insulated.oL.y, insulated.oR.y, insulated.iR.y])
+                            x_bare.append([bare.iL.x, bare.oL.x, bare.oR.x, bare.iR.x])
+                            y_bare.append([bare.iL.y, bare.oL.y, bare.oR.y, bare.iR.y])
+                            i_conductor.append(winding.cable_current * block.current_sign)
+
+                            for strand_gr_nr, strand_gr in halfTurn.strand_groups.items():
+                                for strand_nr, strand in strand_gr.strand_positions.items():
+                                    x_strand.append(strand.x)
+                                    y_strand.append(strand.y)
+                                    i_strand.append(winding.strand_current * block.current_sign)
+                                    strandToHalfTurn.append(idx_half_turn)
+
+    return x_insulated, y_insulated, x_bare, y_bare, i_conductor, x_strand, y_strand, i_strand, strandToHalfTurn
+
+
 class ParserRoxie:
     """
         Class for the roxie parser
@@ -116,6 +188,7 @@ class ParserRoxie:
         designVariables = {}
         scalarVariables = {}
         corners = []
+        notches = []
         bars = []
 
         if iron_content:
@@ -154,15 +227,15 @@ class ParserRoxie:
                                             {**designVariables, **scalarVariables, **data.key_points, **functions})
 
                         elif j[:2] == 'kp':  # key point
-                            allGlobal = {**designVariables, **scalarVariables, **data.key_points, **functions}
+                            all_global = {**designVariables, **scalarVariables, **data.key_points, **functions}
 
                             if '@' in statement[1]:  # then the angle is specified
                                 arg = statement[1].strip(' []').split('@')
-                                A = [evalContent(arg[0].strip(), allGlobal), evalContent(arg[1].strip(), allGlobal)]
+                                A = [evalContent(arg[0].strip(), all_global), evalContent(arg[1].strip(), all_global)]
                                 A = [A[0] * cos(A[1]), A[0] * sin(A[1])]
                             else:  # otherwise coordinates
                                 arg = statement[1].strip(' []').split(',')
-                                A = [evalContent(arg[0].strip(), allGlobal), evalContent(arg[1].strip(), allGlobal)]
+                                A = [evalContent(arg[0].strip(), all_global), evalContent(arg[1].strip(), all_global)]
 
                             data.key_points[state0] = pd.Coord(x=sigDig(A[0]), y=sigDig(A[1]))
 
@@ -178,19 +251,66 @@ class ParserRoxie:
                                     data.hyper_lines[state0] = pd.HyperLine(type='line', kp1=arg0, kp2=arg1)
 
                                 elif lineType[:6] == 'Corner':
+                                    if data.key_points[arg0].y < data.key_points[arg1].y:
+                                        lower = [data.key_points[arg0].x, data.key_points[arg0].y]
+                                        higher = [data.key_points[arg1].x, data.key_points[arg1].y]
+                                    else:
+                                        lower = [data.key_points[arg1].x, data.key_points[arg1].y]
+                                        higher = [data.key_points[arg0].x, data.key_points[arg0].y]
+
                                     if lineType[6:] == 'Out':
                                         arg = arg0 + arg1 + 'Out'  # intersection point
-                                        data.key_points[arg] = pd.Coord(x=data.key_points[arg1].x,
-                                                                        y=data.key_points[arg0].y)
+                                        data.key_points[arg] = pd.Coord(x=lower[0], y=higher[1])
                                     else:
                                         arg = arg0 + arg1 + 'In'
-                                        data.key_points[arg] = pd.Coord(x=data.key_points[arg0].x,
-                                                                        y=data.key_points[arg1].y)
+                                        data.key_points[arg] = pd.Coord(x=higher[0], y=lower[1])
 
                                     corners.append(state0)
                                     data.hyper_lines[state0 + 'b'] = pd.HyperLine(type='line', kp1=arg0, kp2=arg)
                                     data.hyper_lines[state0 + 'a'] = pd.HyperLine(type='line', kp1=arg, kp2=arg1)
 
+                                elif lineType[:5] == 'Notch':
+                                    all_global = {**designVariables, **scalarVariables, **data.key_points, **functions}
+                                    A = [data.key_points[arg0].x, data.key_points[arg0].y]
+                                    B = [data.key_points[arg1].x, data.key_points[arg1].y]
+
+                                    alpha = evalContent(arguments[3].strip(), all_global)
+                                    beta = evalContent(arguments[4].strip(), all_global)
+                                    arg = arg0 + arg1 + 'Notch'  # intersection point
+
+                                    if alpha != 0:
+                                        case = [(abs(alpha) < np.pi / 2) & (alpha > 0),
+                                                (abs(alpha) > np.pi / 2) & (alpha > 0),
+                                                (abs(alpha) < np.pi / 2) & (alpha < 0),
+                                                (abs(alpha) > np.pi / 2) & (alpha < 0)]
+                                        line_a = geometricFunctions.findLineThroughTwoPoints(
+                                            A, [((case[1] | case[2]) - (case[0] | case[3])) * A[1] /
+                                                np.tan((case[1] | case[3]) * np.pi +
+                                                       ((case[0] | case[3]) - (case[1] | case[2])) * alpha) + A[0], 0.])
+                                    else:
+                                        line_a = [0., 1., - A[1]]
+
+                                    if beta != 0:
+                                        case = [(abs(beta) < np.pi / 2) & (beta > 0),
+                                                (abs(beta) > np.pi / 2) & (beta > 0),
+                                                (abs(beta) < np.pi / 2) & (beta < 0),
+                                                (abs(beta) > np.pi / 2) & (beta < 0)]
+                                        line_b = geometricFunctions.findLineThroughTwoPoints(
+                                            B, [((case[1] | case[2]) - (case[0] | case[3])) * B[1] /
+                                                np.tan((case[1] | case[3]) * np.pi +
+                                                       ((case[0] | case[3]) - (case[1] | case[2])) * beta) + B[0], 0.])
+                                    else:
+                                        line_b = [0., 1., - B[1]]
+
+                                    data.key_points[arg] = pd.Coord(x=(line_a[1] * line_b[2] - line_b[1] * line_a[2]) /
+                                                                      (line_a[0] * line_b[1] - line_b[0] * line_a[1]),
+                                                                    y=(line_a[2] * line_b[0] - line_b[2] * line_a[0]) /
+                                                                      (line_a[0] * line_b[1] - line_b[0] * line_a[1]))
+
+                                    notches.append(state0)
+                                    data.hyper_lines[state0 + 'b'] = pd.HyperLine(type='line', kp1=arg0, kp2=arg)
+                                    data.hyper_lines[state0 + 'a'] = pd.HyperLine(type='line', kp1=arg, kp2=arg1)
+
                                 elif lineType == 'Bar':
                                     arg = [arg0 + arg1 + 'a', arg0 + arg1 + 'b']  # rectangle corner points
 
@@ -207,7 +327,8 @@ class ParserRoxie:
                                         else:
                                             h = (B[0] - A[0]) / 2 / math.cos(alpha)
                                     else:
-                                        h = (-1 * (B[0] < A[0]) + (B[0] > A[0])) * \
+                                        h = (((B[0] > A[0]) | ((B[0] == A[0]) & (B[1] < A[1]))) -
+                                             ((B[0] < A[0]) | ((B[0] == A[0]) & (B[1] > A[1])))) * \
                                             evalContent(arguments[3].strip(), {**designVariables, **scalarVariables,
                                                                                **data.key_points, **functions})
 
@@ -218,10 +339,10 @@ class ParserRoxie:
                                         y=sigDig(data.key_points[arg[1]].y + A[1] - B[1]))
 
                                     bars.append(state0)
-                                    data.hyper_lines[state0 + ('c' * (B[0] < A[0]) + 'a' * (B[0] > A[0]))] = \
+                                    data.hyper_lines[state0 + ('c' * (B[0] <= A[0]) + 'a' * (B[0] > A[0]))] = \
                                         pd.HyperLine(type='line', kp1=arg0, kp2=arg[1])
                                     data.hyper_lines[state0 + 'b'] = pd.HyperLine(type='line', kp1=arg[1], kp2=arg[0])
-                                    data.hyper_lines[state0 + ('a' * (B[0] < A[0]) + 'c' * (B[0] > A[0]))] = \
+                                    data.hyper_lines[state0 + ('a' * (B[0] <= A[0]) + 'c' * (B[0] > A[0]))] = \
                                         pd.HyperLine(type='line', kp1=arg[0], kp2=arg1)
 
                                 elif lineType == 'Arc':
@@ -274,9 +395,9 @@ class ParserRoxie:
                                     data.hyper_lines[state0] = pd.HyperLine(type='circle', kp1=arg0, kp2=arg1)
 
                                 elif lineType == 'EllipticArc':
-                                    allGlobal = {**designVariables, **scalarVariables, **data.key_points, **functions}
-                                    arg = [evalContent(arguments[3].strip(), allGlobal),
-                                           evalContent(arguments[4].strip(), allGlobal)]  # axes of the ellipse
+                                    all_global = {**designVariables, **scalarVariables, **data.key_points, **functions}
+                                    arg = [evalContent(arguments[3].strip(), all_global),
+                                           evalContent(arguments[4].strip(), all_global)]  # axes of the ellipse
                                     data.hyper_lines[state0] = \
                                         pd.HyperLine(type='ellipticArc', kp1=arg0, kp2=arg1, arg1=arg[0], arg2=arg[1])
 
@@ -298,8 +419,12 @@ class ParserRoxie:
                             pre_line = []
                             for k in range(len(arguments) - 1):
                                 name = arguments[k].strip()
-                                if name in corners:  # 2 lines are introduced for corners
-                                    arg.extend([name + 'a', name + 'b'])
+                                if name in corners or name in notches:  # 2 lines are introduced for corners and notches
+                                    if (data.hyper_lines[name + 'a'].kp1 in pre_line or
+                                            data.hyper_lines[name + 'a'].kp2 in pre_line):
+                                        arg.extend([name + 'a', name + 'b'])
+                                    else:
+                                        arg.extend([name + 'b', name + 'a'])
                                 elif name in bars:  # 3 lines are introduced for bars
                                     if (data.hyper_lines[name + 'a'].kp1 in pre_line or
                                             data.hyper_lines[name + 'a'].kp2 in pre_line):
@@ -1065,13 +1190,6 @@ class ParserRoxie:
         nColumns = self.cond_parameter.noOfStrandsPerLayer
         nLayers = self.cond_parameter.noOfLayers
 
-        #     print('conductorSigma.name = {}'.format(conductorSigma.name))
-        #     print('wBare = {}'.format(wBare))
-        #     print('hInBare = {}'.format(hInBare))
-        #     print('hOutBare = {}'.format(hOutBare))
-        #     print('wInsulNarrow = {}'.format(wInsulNarrow))
-        #     print('wInsulWide = {}'.format(wInsulWide))
-
         # Define the coefficients of the circle on which the x2 points (bottom-left) of each conductor rest
         # R, x0, and y0 coefficients of the circle, as in: (x-x0)**2 + (y-y0)**2 = R**2
         circle = [radius, x0, y0]
@@ -1083,14 +1201,6 @@ class ParserRoxie:
             print('Initial alpha = {} deg'.format(alpha / math.pi * 180))
             print('Initial phi = {} deg'.format(phi / math.pi * 180))
 
-        # xBare = []
-        # yBare = []
-        # x = []
-        # y = []
-        # xS = []
-        # yS = []
-        # # I = []
-
         # Create coil, pole, layer, and winding keys if they are not present
         if self.block.coil not in self.data.coil.coils:
             self.data.coil.coils[self.block.coil] = pd.Pole()
@@ -1345,7 +1455,7 @@ class ParserRoxie:
         # ((block.pole % 2 == 0) * np.sign(-I[0]) +
         #  (block.pole % 2 != 0) * np.sign(I[0])) * block.radius / 1e3]
 
-        return self.data.coil
+        return self.data.coil.coils if block else self.data.coil
 
     def getCablePositions(self, blocks: Dict[str, pd.Block] = None, cadata: pd.Cadata = None, verbose: bool = False):
         """
@@ -1374,6 +1484,9 @@ class ParserRoxie:
         for no in self.roxieData.coil.blocks:
             self.no = int(no)
             self.block = self.roxieData.coil.blocks[no]
+            self.data.coil.electrical_order.append(
+                pd.Order(coil=self.block.coil, pole=self.block.pole, layer=self.block.layer, winding=self.block.winding,
+                         block=self.no))
 
             # Get desired conductor data
             self.condName = self.block.condname
@@ -1386,7 +1499,7 @@ class ParserRoxie:
 
         # if verbose:
         #     print('Total number of conductors (half-turns): {}'.format(len(self.xPos)))
-        #plt.show()
+        # plt.show()
         return self.data.coil
 
     def getWedgePositions(self, coil: pd.CoilData = None, verbose: bool = False):
